function Out = findSteady(T,ET,TH,om,B,D,DEF,a,b,plotyes,tableyes)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Monetary and Fiscal Policy when People have Finie Lives
% Roger E. A. Farmer and Pawel Zabczyk: August 2023
% Function to Compute the Steady States of a model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N = 2*T-1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define key functions for the technology
Ffr = @(Q) 1./Q - 1 + D;
Ffk = @(Q) (TH./Ffr(Q)).^(1/(1-TH));
Ffy = @(Q) (Ffk(Q)).^TH;
Ffw = @(Q) (1-TH)*(TH./Ffr(Q)).^(TH/(1-TH));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define key functions for preferences
h   = @(Qp,w) sum((om.*w.*Qp));
psi = @(Qp) sum(((B.^ET).*(Qp.^(1-ET))));
wt  = @(Qp) ((B.^ET).*Qp.^(-ET));
C   = @(Qp,w) (wt(Qp).*h(Qp,w))./psi(Qp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define steady state functions

hs = @(Q)  sum((Ffw(Q.*ones(T,1)).*om).*(Q.^((1:T)')));
psis = @(Q) sum((B.^ET).*Q.^((1:T)'*(1-ET)));
wts  = @(Q) (B.^ET).*((Q.^((1:T)'*-ET)));
Cs   = @(Q) wts(Q).*hs(Q)./psis(Q);

hPasts  =  @(Q) tril(ones(T))*((Q.^((1:T)')).*(om.*Ffw(Q.*ones(T,1))));
CPasts  =  @(Q) (tril(ones(T))*((Q.^((1:T)')).*Cs(Q)));

As      =  @(Q) [zeros(1,T);eye(T-1) zeros(T-1,1)]...
    *(hPasts(Q) - CPasts(Q))./(Q.^((0:T-1)'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Excess demand function for goods
Xs = @(Q) Ffy(Q)-sum(Cs(Q)) - D*Ffk(Q) - DEF;
% Excess demand function for assets
XAs = @(Q) (sum(As(Q)) - Ffk(Q));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LQ = linspace(a,b,10000);
Q  = exp(LQ);

Qa = Q(1:end-1);
Qb = Q(2:end);
Q_init = Q(sign(Xs(Qa)) ~= sign(Xs(Qb)));
options = optimoptions('fsolve','Display','none','Diagnostics','off');
Qs = uniquetol(fsolve(Xs,Q_init,options),1e-6);
disp(['Zeros of Excess Demand for Goods   : ' newline num2str(Qs,3) newline])
disp(['Government Debt at the Steady State: ' newline num2str(round(100*XAs(Qs)),2) newline])

% Qa_init = Q(sign(XAs(Qa)) ~= sign(XAs(Qb)));
% Qas = uniquetol(fsolve(XAs,Qa_init,options),1e-3);

if plotyes == 1
    plot(LQ,Xs(Q),'LineWidth',2)
    hold on
    plot(LQ,XAs(Q)./max(XAs(Q)),'LineWidth',2)
    plot(LQ,Q*0,'k--','LineWidth',1)
    hold off
    axis tight
    xlim([a,max(log(Qs))*1.03]);
    hold off
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the jacobian of the excess demand function

Jps = @(Q) (repmat(Q.^((0:T-1)'),1,T)).*tril(ones(T));

Ffw_Qs      = @(Q) diag(TH^(1/(1-TH))*(1./(Q*ones(T,1))-1+D)...
    .^(1/(TH-1))./(Q*ones(T,1)).^2);

if TH ==0 || D == 1
    Ffk_Qs = @(Q) 0;
    Ffy_Qs = @(Q) 0;
else
    Ffk_Qs      = @(Q) diag((1/(1-TH))*((1./(Q)-1+D)/TH)...
        .^((2-TH)/(TH-1))./(TH*Q.^2));
    Ffy_Qs      = @(Q) TH*Ffk(Q).^(TH-1)*Ffk_Qs(Q);
end


h_Qs        = @(Q) (om.*Ffw(Q*ones(T,1))).'*Jps(Q) + (om.*(Q.^((1:T)'))).'*Ffw_Qs(Q);
psi_Qs      = @(Q) ((1-ET)*(B.^ET).*((Q.^((1:T)')).^-ET)).'*Jps(Q);
wt_Qs       = @(Q) -diag(ET*(wt((Q.^((1:T).')))./(Q.^((1:T).'))).')*Jps(Q);
Term_1      = @(Q)  hs(Q)./psis(Q);
Term_1_Q    = @(Q) (h_Qs(Q)/psis(Q)-hs(Q)*psi_Qs(Q)/psis(Q).^2);
C_Qs        = @(Q) wts(Q).*Term_1_Q(Q) + Term_1(Q).*wt_Qs(Q);

x1          = @(Q) [zeros(1,T-1) Ffk_Qs(Q) zeros(1,N-T)]; % Future capital;
x2          = @(Q) [zeros(1,T) Ffk_Qs(Q) zeros(1,N-T-1)]; % Current capital;
x3          = @(Q) [zeros(1,T-1) Ffy_Qs(Q) zeros(1,N-T)]; % Current GDP

n = numel(Qs);
X_Q = cell(n,1);
J1 = cell(n,1);
J2 = cell(n,1);
aevs = cell(n,1);
AA = cell(n,1);
BB = cell(n,1);
QQ = cell(n,1);
ZZ = cell(n,1);
nfstable = cell(n,1);
Ac = cell(n,1);
IndetNum = zeros(n,1);

if tableyes == 1
    fprintf(  '\n');
    disp('______________________________________________________________')
end
for j = 1:n
    Ac{j} = zeros(T,1);
    X_Q{j} = zeros(1,N);
    J1{j} = sparse(zeros(N-1,N-1));
    J2{j} = sparse(zeros(N-1,N-1));
    S = zeros(1,N);
    temp1 = C_Qs(Qs(j));
    for t = 1:T
        temp = temp1(t,:);
        S(1,T-t+1:2*T-t) = S(1,T-t+1:2*T-t) + temp;
    end
    X_Q{j} = S + x2(Qs(j)) - (1-D)*x1(Qs(j)) - x3(Qs(j));
    J1{j}(1,:) = X_Q{j}(1,1:end-1);
    J1{j}(2:end,2:end) = sparse(eye(N-2));
    J2{j}(1,end) = - X_Q{j}(1,end);
    J2{j}(2:end,1:end-1) = sparse(eye(N-2));
    [AA{j},BB{j},QQ{j},ZZ{j}] = qz(full(J2{j}),full(J1{j}));
    aevs{j} = abs(diag(AA{j}./BB{j}));
    nfstable{j}  = sum(aevs{j}<1);

    IndetNum(j) =  T-1 - nfstable{j};
    if tableyes == 1
        disp(['Value of Excess Demand at Steady State where Q = ' ...
            num2str(Qs(j),'%.3f') ' is ' num2str(Xs(Qs(j)),'%.1f')])
        disp(['Value of Outside Asset Demand is ' ...
            num2str(XAs(Qs(j))*100,'%.2f') ' percent of GDP'])
        
        disp([ 'Degree of indeterminacy is ' num2str(IndetNum(j)) ])
        disp('______________________________________________________________')
    end
end

Out.Qs = Qs;
Out.Qa = Qa;
Out.IndetNum = IndetNum;

end