function [T,ET,TH,om,B,D,DEF,a,b] = pickParams(modelType)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Available model types
% modelType = 'KehoeLevine'
% modelType = 'TwoPeriod'
% modelType = 'ThreePeriod'
% modelType = 'GuvenenProduction'
% modelType = 'GuvenenEndowment'
% modelType = 'FourPeriod'
% modelType = 'TPeriodProduction'
% om is 1 x T vector of endowment weights
% B is a 1 x T vector of utility weights
% DEF is the budget deficit
% ET is the intertemporal elasticity of substitution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DTA         = exp([9.703325394	9.811688551	9.898490588	9.973850019	...
    10.03586359	10.09093885	10.13974711	10.18394484	10.2227228	...
    10.25816555	10.29195535	10.32096127	10.34686836	10.37245815	...
    10.39587656	10.41480585	10.43416139	10.44915827	10.46393585	...
    10.47578516	10.48750685	10.49925369	10.50725364	10.51323138	...
    10.52036662	10.52500553	10.52698263	10.52461362	10.52239087	...
    10.5192433	10.51468606	10.498851	10.47198209	10.44670331	...
    10.4198182	10.39065399]);
INIT_PER    = exp(9.703325394 - (9.811688551-9.703325394)*fliplr(1:7));
FIN_PER     = exp(10.39065399 + (10.39065399-10.4198182)*(1:6));

W_Guvenen  = [INIT_PER,DTA,FIN_PER,sum([INIT_PER,DTA,FIN_PER])...
    *0.00137*ones(1,13)/13];

% The Guvenen profile starts at age 25 and goes to 60
% By linearly interpolating we arrive at a profile from age 18
% (i.e. adulthood) to 66 i.e. retirement For the remaining 13 years
% we assume agents are entitled to social security benefits totaling
% 0.137% (i.e. a tenth of a percent) of GDP.  This implies that
% every year they receive 1/13th of that fraction to ensure total
% benefit payments add up to the correct fraction

switch modelType

    case 'KehoeLevine'
        T = 3;        
        D = 1;
        TH = 0;
        B = [2 2 1];
        om = [3 15 2]/20;
        ET = 0.2;
        DEF = 0;        
        a = -5;
        b = 5;        

    case 'TwoPeriod'
        T = 2;        
        D = 1;
        TH = 0;
        if exist('B','var') == 0 || isempty(B)
            B = [1 1];            
        end
        if exist('om','var') == 0 || isempty(om)
            om = [1 0.5];
        end
        if exist('ET','var') == 0  || isempty(ET)
            ET = 0.2;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0;
        end
        om = om/sum(om);        
        a = -5;
        b = 5;        

    case 'ThreePeriod'
        T = 3;        
        D = 1;
        TH = 0;
        if exist('B','var') == 0 || isempty(B)
            B = [1 1 1];            
        end
        if exist('om','var') == 0 || isempty(om)
            om = [1 0.25 0.25];
        end
        if exist('ET','var') == 0  || isempty(ET)
            ET = 1;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0;
        end
        om = om/sum(om);        
        a = -5;
        b = 5;        

    case 'FourPeriod'
        T = 4;
        N = 2*T-1;
        D = 0.9;
        TH = 0.3;
        if exist('B','var') == 0 || isempty(B)
            B = [1 1 1 1];            
        end
        if exist('om','var') == 0 || isempty(om)
            om = [1 0.9 0.9^2 0.9^3];
        end
        if exist('ET','var') == 0  || isempty(ET)
            ET = 0.8;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0;
        end
        om = om/sum(om);       
        a = -4;
        b = log(0.99/(1-D));    

    case 'GuvenenProduction'
        T = 62;        
        D_annual = 0.15;
        D = 1 - (1-D_annual)^round(60/T);        
        if exist('B','var') == 0 || isempty(B)
            Ba = 0.9995;
            B = Ba;
        end
        B = B.^(0:T-1);
        if exist('ET','var') == 0  || isempty(ET)
            ET = .03;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0.0;
        end
        om = W_Guvenen/sum(W_Guvenen);
        TH = 0.3;
        a = -0.05;
        b = .09;        

    case 'GuvenenEndowment'
        T = 62;        
        D = 1;
        if exist('B','var') == 0 || isempty(B)
            Ba = 0.953;
            B = Ba;
        end
        B = B.^(0:T-1);
        if exist('ET','var') == 0  || isempty(ET)
            ET = 1/6;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0;
        end
        TH = 0;
        a = -0.3;
        b = 0.7;
        om = W_Guvenen/sum(W_Guvenen);        
        
    case 'TPeriodProduction'
        T = 12;                              % Amend this as needed
        D_annual = 0.15;
        D = 1 - (1-D_annual)^round(62/T);        
        if exist('B','var') == 0 || isempty(B)
            B = 0.9;
        end
        B = (B^round(62/T)).^(0:T-1);
        disp(B)
        if exist('ET','var') == 0  || isempty(ET)
            ET = 0.05;
        end
        if exist('DEF','var') == 0  || isempty(DEF)
            DEF = 0.0;
        end
        om = ones(1,T)*1.001;
        Retire = (65-20)/(85-20);
        T_Retire = floor(Retire*T);
        slope = 0.05;
        om(1) = 1.1;
        for t = 2:T_Retire
            om(t) = om(1) + slope*(t-1);
        end
        om = log(om);
        figure(1)
        om = om/sum(om);
        plot(om);        
        TH = 0.3;
        a = -4;
        b = 0.4;        

end

